﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Text.RegularExpressions;
using PR_check_UI.Properties;
using System.Xml.Linq;
using System.Xml;
using System.Xml.Schema;

namespace PR_check_UI
{
    public class History
    {
        #region Single

        private static History instance;

        /// <summary>
        /// 
        /// </summary>
        public static History Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new History();
                }

                return instance;
            }
        }

        /// <summary>
        /// Ctor
        /// </summary>
        private History()
        {
        } 

        #endregion

        #region Members
        List<HistoryItem>   LstItems;
        XmlDocument         Document;
        bool                XMLValidationPassed;
        #endregion

        #region Internal Methods

        /// <summary>
        /// 
        /// </summary>
        internal bool Init()
        {
            bool LocalLoadException = false;

            // Create XmlDocument
            Document = new XmlDocument();

            // Load Schema
            Document.Schemas.Add(null, @"http://www.shvavim.net/or/PRChecker/History.xsd");

            // Load XML file
            try
            {
                Document.Load(@Settings.Default.HistoryXmlFile);

                // Set Validation to true
                XMLValidationPassed = true;

                // Validate XML with the schema
                // Change validation flag if invalid
                Document.Validate(new ValidationEventHandler(this.ValidationHandler));

                // Load items list if validate passed
                if (XMLValidationPassed)
                {
                    LstItems = (from CurrentSite
                           in Document.GetElementsByTagName("Site").Cast<XmlNode>()

                                select new HistoryItem()
                                {
                                    URL = CurrentSite.Attributes["URL"].Value,
                                    Rank = Int32.Parse(CurrentSite.Attributes["Rank"].Value),
                                    Date = DateTime.Parse(CurrentSite.Attributes["Date"].Value)
                                }).ToList();
                }
            }
            catch
            {
                LocalLoadException = true;
            }
            finally
            {
            }

            return (XMLValidationPassed && (!LocalLoadException));
        }

        /// <summary>
        /// When XML validation failed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="vargs"></param>
        private void ValidationHandler(object sender, ValidationEventArgs vargs)
        {
            XMLValidationPassed = false;
        }

        /// <summary>
        /// Get Sites list.
        /// Distinct by site url
        /// </summary>
        internal List<HistoryItem> DistinctSites
        {
            get
            {
                return LstItems.Distinct().ToList();
            }
        }

        /// <summary>
        /// Return all Items with given URL order by date
        /// </summary>
        /// <param name="URL"></param>
        /// <returns></returns>
        internal List<HistoryItem> GetItemsByItemURL(string URL)
        {
            List<HistoryItem> lst = LstItems.Where((x, y) => string.Equals(x.URL, URL)).ToList();
            lst.Sort((x, y) => DateTime.Compare(x.Date, y.Date));

            return lst;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="Site"></param>
        internal void AddSite(HistoryItem Site)
        {
            this.AddSiteToXML(Site);
            this.AddSiteToLocalList(Site);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="URL"></param>
        internal void RemoveItemByURL(string URL)
        {
            this.RemoveFromXML(URL);
            this.RemoveFromLocalList(URL);
        } 

        #endregion

        #region Private Methods

        /// <summary>
        /// 
        /// </summary>
        private void Update()
        {
            Document.Load(@Settings.Default.HistoryXmlFile);

            LstItems = (from CurrentSite
                   in Document.GetElementsByTagName("Site").Cast<XmlNode>()

                        select new HistoryItem()
                        {
                            URL = CurrentSite.Attributes["URL"].Value,
                            Rank = Int32.Parse(CurrentSite.Attributes["Rank"].Value),
                            Date = DateTime.Parse(CurrentSite.Attributes["Date"].Value)
                        }).ToList();
        }

        /// <summary>
        /// Same result as call to 'Update' method
        /// </summary>
        /// <param name="URL"></param>
        private void RemoveFromLocalList(string URL)
        {
            // Set all other list elements as the new list
            LstItems = LstItems.Where((x) => !string.Equals(x.URL, URL)).ToList();

            // OR:
            // Get all list elements with this URL
            // And remove them from original list
            /*
            List<HistoryItem> ItemsToRemove = new List<HistoryItem>();
            ItemsToRemove = LstItems.Where((x) => string.Equals(x.URL, URL)).ToList();

            foreach (HistoryItem CurrentItem in ItemsToRemove)
            {
                LstItems.Remove(CurrentItem);
            }
            */
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="URL"></param>
        private void RemoveFromXML(string URL)
        {
            //TODOO: change to LINQ
            XmlNodeList NList = Document.GetElementsByTagName("Site");
            List<XmlNode> NodesToRemove = new List<XmlNode>();

            foreach (XmlNode CurrentNode in NList)
            {
                if (CurrentNode.Attributes["URL"].Value.ToString() == URL)
                {
                    NodesToRemove.Add(CurrentNode);
                }
            }

            foreach (XmlNode CurrentNode in NodesToRemove)
            {
                CurrentNode.ParentNode.RemoveChild(CurrentNode);
            }

            Document.Save(@Settings.Default.HistoryXmlFile);
        }

        /// <summary>
        /// Same result as call to 'Update' method
        /// </summary>
        /// <param name="Site"></param>
        private void AddSiteToLocalList(HistoryItem Site)
        {
            this.LstItems.Add(Site);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="Site"></param>
        private void AddSiteToXML(HistoryItem Site)
        {
            XmlElement NewXMLNode = Document.CreateElement("Site");
            XmlAttribute URLAttirbute = Document.CreateAttribute("URL");
            XmlAttribute RankAttirbute = Document.CreateAttribute("Rank");
            XmlAttribute DateAttirbute = Document.CreateAttribute("Date");

            URLAttirbute.Value = Site.URL;
            RankAttirbute.Value = Site.Rank.ToString();
            DateAttirbute.Value = Site.Date.ToString();

            NewXMLNode.Attributes.Append(URLAttirbute);
            NewXMLNode.Attributes.Append(RankAttirbute);
            NewXMLNode.Attributes.Append(DateAttirbute);

            Document.GetElementsByTagName("History")[0].AppendChild(NewXMLNode);

            Document.Save(@Settings.Default.HistoryXmlFile);
        } 

        #endregion
    }
}
